//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include "jetfight.h"
#include <jrnd.h>

//                       ***************************
//                       ***PLAYER SYSTEM CLASSES***
//                       ***************************

inline void PlayerSystem::Calc() { //calcs player's new position
  track.x += sin(track.angle * a2rad) * track.speed;
  track.y -= cos(track.angle * a2rad) * track.speed;

  if (track.x < 0.0)
    track.x = 0.0;
  else if (track.x > unitx - 32.0)
    track.x = unitx - 32.0;
  if (track.y < 0.0)
    track.y = 0.0;
  else if (track.y > unity - 32.0)
    track.y = unity - 32.0;

  if (track.angle < 0.0)
    track.angle += 360.0;
  else if (track.angle > 360.0)
    track.angle -= 360.0;

  if (!autopilot) {
    if ((track.angle < 90.0 + stats.turnrate) && (track.angle > 90.0 - stats.turnrate))
      track.angle = 90.0;
    else if ((track.angle < 180.0 + stats.turnrate) && (track.angle > 180.0 - stats.turnrate))
      track.angle = 180.0;
    else if ((track.angle < 270.0 + stats.turnrate) && (track.angle > 270.0 - stats.turnrate))
      track.angle = 270.0;
    else if ((track.angle < 0.0 + stats.turnrate) || (track.angle > 360.0 - stats.turnrate))
      track.angle = 0.0;
  }
}

void PlayerSystem::PlayerAI() { //autopilot
  navactive = 1;
  autopilot = 0;
  tempdist = FindDistance(track.x, track.y, navdata[onpoint].x, navdata[onpoint].y);
  tempangle = FindAngle(track.x, track.y, navdata[onpoint].x, navdata[onpoint].y);
  tempdir = FindDirection(track.angle, tempangle);
  if (tempdist > 192.0)
    Accelerate();
  else
    Deccelerate();
  if (fabs(tempdir) < stats.turnrate)
    track.angle = tempangle;
  else if (tempdir > 0)
    track.angle += stats.turnrate; //TurnRight();
  else if (tempdir < 0)
    track.angle -= stats.turnrate; //TurnLeft();
  autopilot = 1;
}

PlayerSystem::PlayerSystem() {  //creates player
  waypoints = 0;
}

PlayerSystem::~PlayerSystem() { //destroys player
}

int PlayerSystem::Init(int type, PlayerTrack track2, NavSystemData navdata2[10], PlayerAmmo ammo2, unsigned char waypoints2) { //loads data for player
  switch (type) {
    case 0: //a-10
      stats.image = a10;
      stats.minspeed = 4.0;
      stats.maxspeed = 16.0;
      stats.turnrate = 6.0;
      stats.life = 100.0;
      stats.accel = 0.5;
      ammo.bulletdamage = 40.0;
      ammo.bombdamage = 50.0;
      ammo.missiledamage = 100.0;
      break;
    case 1: //f-15
      stats.image = f15;
      stats.minspeed = 6.0;
      stats.maxspeed = 32.0;
      stats.turnrate = 12.0;
      stats.life = 40.0;
      stats.accel = 1.0;
      ammo.bulletdamage = 20.0;
      ammo.bombdamage = 10.0;
      ammo.missiledamage = 120.0;
      break;
    default: //a-10 if nothing else works
      stats.image = a10;
      stats.minspeed = 4.0;
      stats.maxspeed = 16.0;
      stats.turnrate = 6.0;
      stats.life = 100.0;
      stats.accel = 0.5;
      ammo.bulletdamage = 40.0;
      ammo.bombdamage = 30.0;
      ammo.missiledamage = 100.0;
      break;
  }

  playertype = type;

  ammo = ammo2;
  track = track2;
  track.speed = stats.minspeed;

  navactive = 0;
  waypoints = waypoints2;
  onpoint = 0;

  int c;
  for (c = 0; c < 10; c++)
    navdata[c] = navdata2[c];

  return 0;
}

void PlayerSystem::SetPos(short int x, short int y) {
  track.x = x * 32;
  track.y = y * 32;
}

void PlayerSystem::TurnLeft() {
  if (!autopilot) track.angle -= stats.turnrate;
}

void PlayerSystem::TurnRight() {
  if (!autopilot) track.angle += stats.turnrate;
}

void PlayerSystem::Accelerate() {
  if (!autopilot) {
    track.speed += stats.accel;
    if (track.speed > stats.maxspeed)
      track.speed = stats.maxspeed;
  }
}

void PlayerSystem::Deccelerate() {
  if (!autopilot) {
    track.speed -= stats.accel;
    if (track.speed < stats.minspeed)
      track.speed = stats.minspeed;
  }
}

void PlayerSystem::ActivateNavSystem() {
  if (waypoints)
    if (navactive) {
      navactive = 0;
    } else {
      navactive = 1;
      Sound.NavSystemOn(128);
    }
  else
    Video.DrawMessage(scrx - 8 * 19, 0, 15, "No Waypoints!!!");
}

void PlayerSystem::NextWaypoint() {
  if (navactive) {
    onpoint++;
    if (onpoint >= waypoints)
      onpoint = 0;
    Sound.Blip(128);
  }
}

void PlayerSystem::EnemyKilled(unsigned char index) {//when an enemy dies, modify nav points
  for (char c=0; c<waypoints; c++) {
    if (navdata[c].targ > index) //if the index is higher
      navdata[c].targ--;         //reduce it by one
    else if(navdata[c].targ == index) {//if THIS enemy died
      for (char d=c; d<waypoints; d++) //delete and shift array
        navdata[c] = navdata[d];
      waypoints--;               //we have 1 less waypoint now;
      if (!waypoints) {          //if 0 waypoints
        navactive = 0;           //Turn off nav and autopilot
        autopilot = 0;
      }
    }
  }
}

void PlayerSystem::PrevWaypoint() {
  if (navactive) {
    onpoint--;
    if (onpoint < 0)
      onpoint = waypoints;
    Sound.Blip(128);
  }
}

void PlayerSystem::SetAutopilot() {
  if (waypoints)
    if (autopilot) {
      autopilot = 0;
    } else {
      autopilot = 1;
      Sound.AutopilotOn(128);
      if (!navactive) {
        navactive = 1;
        Sound.NavSystemOn(128);
      }
      //Autopilot activated sound
    }
  else
    Video.DrawMessage(scrx - 8 * 19, 0, 15, "No Waypoints!!!");
}

void PlayerSystem::Shoot(int cmd) {
  if (cmd && ammo.bullets > 0) {
    Sound.MachineGun(128, 1);
    ammo.bullets -= 8;
    if (ammo.bullets < 0)
      ammo.bullets = 0;
    //Make a Bullet
    //Greys from 114 to 121
    Weapons.MakeNewPlayer(sin(track.angle * a2rad) * 16.0 + track.x + 16.0, //x
                          -cos(track.angle * a2rad) * 16.0 + track.y + 16.0,//y
                          track.angle, track.speed + 20,    //angle, speed
                          115, 15, ammo.bulletdamage);//color, life, damage
  } else {
    Sound.MachineGun(128, 0);
  }
}

int PlayerSystem::Damage(float x, float y, float damage) { //Did we get hurt?
  if (FindDistance(x, y, track.x, track.y) < 16) {
    stats.life -= damage;
    if (stats.life < 0)
      Sound.Explosion(128);
    return 1;
  }

  return 0;
}

void PlayerSystem::SetView() {
  if (autopilot)
    PlayerAI();
  Calc();
  Video.SetView(track.x, track.y);
}

void PlayerSystem::Update() {
  if (navactive)
    Video.DrawNavSystem(track, navdata[onpoint]); //The navdata.x/.y also
                                                  //get updated at this point

  Video.DrawObject(stats.image, track.x, track.y, track.angle);

  Video.DrawInterface(stats, ammo, track);
  if (autopilot)
    Video.DrawMessage(scrx - 8 * 19, 0, 15, "AUTOPILOT ACTIVATED");

//Make a Particle
//Oranges from from 160 to 175
//Whites from 114 to 121
  float tempangle = (track.angle - 180.0);
  for (char c = 0; c < 2; c++)
    Parts.MakeNew(sin(tempangle * a2rad) * 16.0 + track.x + 16.0, //x
                  -cos(tempangle * a2rad) * 16.0 + track.y + 16.0,//y
                  tempangle + getrnd(10, -10),            //angle
                  -track.speed * 0.3 + getrndf(1.0, -1.0),//speed
                  getrnd(121, 114), 15);                  //color, life

  Sound.Jet(128, 1, 1000 + int(track.speed) * 50);
}

//        ***EDITOR PROCS

void PlayerSystem::Load(PACKFILE *mapdata) { //loads Player from a map
  pack_fread(&playertype, sizeof(unsigned char), mapdata);
  pack_fread(&track, sizeof(PlayerTrack), mapdata);
  pack_fread(&ammo, sizeof(PlayerAmmo), mapdata);
  pack_fread(&waypoints, sizeof(unsigned char), mapdata);
  pack_fread(navdata, sizeof(NavSystemData) * 10, mapdata);
  Init(playertype, track, navdata, ammo, waypoints);
}

void PlayerSystem::Save(PACKFILE *mapdata) { //saves Player to a map
  pack_fwrite(&playertype, sizeof(unsigned char), mapdata);
  pack_fwrite(&track, sizeof(PlayerTrack), mapdata);
  pack_fwrite(&ammo, sizeof(PlayerAmmo), mapdata);
  pack_fwrite(&waypoints, sizeof(unsigned char), mapdata);
  pack_fwrite(navdata, sizeof(NavSystemData) * 10, mapdata);
}

void PlayerSystem::Draw(BITMAP *BUFFER, short int viewx, short int viewy) {
  //ANGLE IN 360 DEGREE FORMAT!!!
  int x, y;
  x = int(track.x);
  y = int(track.y);
  if (x > viewx - 32 && x < viewx + mscrx && y > viewy - 32 && y < viewy + mscry) {
    tempangle = track.angle * 256.0 / 360.0;
    rotate_sprite(BUFFER, (BITMAP *)data[stats.image].dat, x - viewx, y - viewy, ftofix(tempangle));
  }
  textout_centre(BUFFER, font, "Player", x - viewx + 16, y - viewy + 12, 15);
}

void PlayerSystem::SetNavData(NavSystemData nd[10], unsigned char numpoints) {//saves navdata
  for (char c=0; c < 10; c++)
    navdata[c] = nd[c];

  waypoints = numpoints;
}

void PlayerSystem::GetData(PlayerTrack &track2, PlayerAmmo &ammo2, unsigned char &type2, NavSystemData navdata2[10], unsigned char &waypoints2) {
  track2 = track;
  ammo2 = ammo;
  type2 = playertype;
  for (char c=0; c < 10; c++)
    navdata2[c] = navdata[c];
  waypoints2 = waypoints;
}

void PlayerSystem::SetData(PlayerTrack track2, PlayerAmmo ammo2, unsigned char type2, NavSystemData navdata2[10], unsigned char waypoints2) {
  Init(type2, track2, navdata2, ammo2, waypoints2);
}
